"""
Tests for the Sphinx builder.
"""

from collections.abc import Callable
from pathlib import Path

from sphinx.testing.util import SphinxTestApp

import sphinx_notionbuilder


def test_builder_meta(
    make_app: Callable[..., SphinxTestApp],
    tmp_path: Path,
) -> None:
    """
    Test the metadata of the Notion builder.
    """
    builder_cls = sphinx_notionbuilder.NotionBuilder
    assert builder_cls.name == "notion"
    assert builder_cls.out_suffix == ".json"
    assert (
        builder_cls.default_translator_class
        == sphinx_notionbuilder.NotionTranslator
    )

    srcdir = tmp_path / "src"
    srcdir.mkdir()
    (srcdir / "conf.py").touch()
    app = make_app(srcdir=srcdir)
    setup_result = sphinx_notionbuilder.setup(app=app)
    assert setup_result == {"parallel_read_safe": True}



"""
Configuration for pytest.
"""

import pytest
from beartype import beartype

pytest_plugins = "sphinx.testing.fixtures"  # pylint: disable=invalid-name


def pytest_collection_modifyitems(items: list[pytest.Item]) -> None:
    """
    Apply the beartype decorator to all collected test functions.
    """
    for item in items:
        # All our tests are functions, for now
        assert isinstance(item, pytest.Function)
        item.obj = beartype(obj=item.obj)



"""
Sphinx Notion Builder.
"""

from beartype import beartype
from sphinx.application import Sphinx
from sphinx.builders.text import TextBuilder
from sphinx.util.typing import ExtensionMetadata
from sphinx.writers.text import TextTranslator


@beartype
class NotionTranslator(TextTranslator):
    """
    Translate docutils nodes to Notion JSON.
    """


@beartype
class NotionBuilder(TextBuilder):
    """
    Build Notion-compatible documents.
    """

    name = "notion"
    out_suffix = ".json"
    default_translator_class: type[NotionTranslator] = NotionTranslator


@beartype
def setup(app: Sphinx) -> ExtensionMetadata:
    """
    Add the builder to Sphinx.
    """
    app.add_builder(builder=NotionBuilder)
    return {"parallel_read_safe": True}
